@@ -925,15 +925,15 @@ static gboolean
 gth_image_viewer_page_real_can_view (GthViewerPage *base,
 				     GthFileData   *file_data)
 {
-	GthImageViewerPage *self;
-
-	self = (GthImageViewerPage*) base;
 	g_return_val_if_fail (file_data != NULL, FALSE);
 
 	return _g_mime_type_is_image (gth_file_data_get_mime_type (file_data));
 }
 
 
+#define N_PRELOADERS 4
+
+
 static void
 gth_image_viewer_page_real_view (GthViewerPage *base,
 				 GthFileData   *file_data)
@@ -941,9 +941,9 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	GthImageViewerPage *self;
 	GthFileStore       *file_store;
 	GtkTreeIter         iter;
-	GthFileData        *next_file_data = NULL;
-	GthFileData        *next2_file_data = NULL;
-	GthFileData        *prev_file_data = NULL;
+	int                 i;
+	GthFileData        *next_file_data[N_PRELOADERS];
+	GthFileData        *prev_file_data[N_PRELOADERS];
 	int                 window_width;
 	int                 window_height;
 
@@ -967,22 +967,28 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	self->priv->file_data = gth_file_data_dup (file_data);
 	self->priv->image_changed = FALSE;
 
+	for (i = 0; i < N_PRELOADERS; i++) {
+		next_file_data[i] = NULL;
+		prev_file_data[i] = NULL;
+	}
+
 	file_store = gth_browser_get_file_store (self->priv->browser);
 	if (gth_file_store_find_visible (file_store, self->priv->file_data->file, &iter)) {
-		GtkTreeIter iter2;
-		GtkTreeIter iter3;
+		GtkTreeIter next_iter;
 
-		iter2 = iter;
-		if (gth_file_store_get_next_visible (file_store, &iter2))
-			next_file_data = gth_file_store_get_file (file_store, &iter2);
+		next_iter = iter;
+		for (i = 0; i < N_PRELOADERS; i++) {
+			if (! gth_file_store_get_next_visible (file_store, &next_iter))
+				break;
+			next_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+		}
 
-		iter3 = iter2;
-		if (gth_file_store_get_next_visible (file_store, &iter3))
-			next2_file_data = gth_file_store_get_file (file_store, &iter3);
-
-		iter2 = iter;
-		if (gth_file_store_get_prev_visible (file_store, &iter2))
-			prev_file_data = gth_file_store_get_file (file_store, &iter2);
+		next_iter = iter;
+		for (i = 0; i < N_PRELOADERS; i++) {
+			if (! gth_file_store_get_prev_visible (file_store, &next_iter))
+				break;
+			prev_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+		}
 	}
 
 	gtk_window_get_size (GTK_WINDOW (self->priv->browser),
@@ -992,9 +998,14 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	gth_image_preloader_load (self->priv->preloader,
 				  self->priv->file_data,
 				  (gth_image_prelaoder_get_load_policy (self->priv->preloader) == GTH_LOAD_POLICY_TWO_STEPS) ? MAX (window_width, window_height) : -1,
-				  next_file_data,
-				  next2_file_data,
-				  prev_file_data,
+				  next_file_data[0],
+				  next_file_data[1],
+				  next_file_data[2],
+				  next_file_data[3],
+				  prev_file_data[0],
+				  prev_file_data[1],
+				  prev_file_data[2],
+				  prev_file_data[3],
 				  NULL);
 }
 
@@ -1104,7 +1115,7 @@ image_saved_cb (GthFileData *file_data,
 	if (data->func != NULL)
 		(data->func) ((GthViewerPage *) self, self->priv->file_data, error, data->user_data);
 	else if (error != NULL)
-		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (self->priv->browser), _("Could not save the file"), &error);
+		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (self->priv->browser), _("Could not save the file"), error);
 
 	if (! error_occurred) {
 		GFile *folder;
@@ -1293,6 +1304,9 @@ _gth_image_viewer_page_set_image (GthImageViewerPage *self,
 	int          height;
 	char        *size;
 
+	if (image == NULL)
+		return;
+
 	gth_image_viewer_set_surface (GTH_IMAGE_VIEWER (self->priv->viewer), image, -1, -1);
 
 	file_data = gth_browser_get_current_file (GTH_BROWSER (self->priv->browser));
@@ -1488,10 +1502,17 @@ gth_image_viewer_page_set_image (GthImageViewerPage *self,
 			 	 cairo_surface_t    *image,
 			 	 gboolean            add_to_history)
 {
+	if (gth_image_viewer_page_get_image (self) == image)
+		return;
+
 	if (add_to_history)
 		gth_image_history_add_image (self->priv->history, image, TRUE);
+
 	_gth_image_viewer_page_set_image (self, image, TRUE);
 	self->priv->image_changed = TRUE;
+
+	if (add_to_history)
+		gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 }
 
 
@@ -1543,7 +1564,7 @@ add_non_content_width (GthImageViewerPage *self,
 {
 	int width = 0;
 
-	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
+	if ((non_content != NULL) && gtk_widget_get_mapped (non_content)) {
 		GtkAllocation allocation;
 
 		gtk_widget_get_allocation (non_content, &allocation);
@@ -1560,7 +1581,7 @@ add_non_content_height (GthImageViewerPage *self,
 {
 	int height = 0;
 
-	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
+	if ((non_content != NULL) && gtk_widget_get_mapped (non_content)) {
 		GtkAllocation allocation;
 
 		gtk_widget_get_allocation (non_content, &allocation);
