@@ -201,7 +201,7 @@ for_each_child_data_free (ForEachChildData *fec)
 		return;
 
 	g_object_unref (fec->base_directory);
-	if (fec->already_visited)
+	if (fec->already_visited != NULL)
 		g_hash_table_destroy (fec->already_visited);
 	clear_child_data (&(fec->current));
 	if (fec->to_visit != NULL) {
