@@ -72,6 +72,7 @@ typedef struct {
 	int                  ival;
 	GFile               *file;
 	GthFileData         *file_data;
+	int                  position;
 } GthFileListOp;
 
 
@@ -986,7 +987,8 @@ _gth_file_list_get_metadata (GthFileList *file_list,
 
 static void
 gfl_add_files (GthFileList *file_list,
-	       GList       *files)
+	       GList       *files,
+	       int          position)
 {
 	GthFileStore *file_store;
 	GList        *scan;
@@ -1038,19 +1040,21 @@ gfl_add_files (GthFileList *file_list,
 	}
 	g_free (cache_base_uri);
 
-	gth_file_store_exec_add (file_store);
+	gth_file_store_exec_add (file_store, position);
 	_gth_file_list_update_pane (file_list);
 }
 
 
 void
 gth_file_list_add_files (GthFileList *file_list,
-			 GList       *files)
+			 GList       *files,
+			 int          position)
 {
 	GthFileListOp *op;
 
 	op = gth_file_list_op_new (GTH_FILE_LIST_OP_TYPE_ADD_FILES);
 	op->file_list = _g_object_list_ref (files);
+	op->position = position;
 	_gth_file_list_queue_op (file_list, op);
 }
 
@@ -1202,7 +1206,7 @@ gfl_set_files (GthFileList *file_list,
 
 	gth_file_store_clear ((GthFileStore*) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view)));
 	g_hash_table_remove_all (file_list->priv->thumb_data);
-	gfl_add_files (file_list, files);
+	gfl_add_files (file_list, files, -1);
 }
 
 
@@ -1589,6 +1593,7 @@ thumbnail_job_ready_cb (GObject      *source_object,
 	if ((! success && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 	    || file_list->priv->cancelling)
 	{
+		_g_object_unref (pixbuf);
 		thumbnail_job_free (job);
 		return;
 	}
@@ -1781,7 +1786,6 @@ _gth_file_list_get_visibles (GthFileList *file_list)
 static void
 _gth_file_list_update_next_thumb (GthFileList *file_list)
 {
-	GthFileStore  *file_store;
 	int            pos;
 	int            first_pos;
 	int            last_pos;
@@ -1808,8 +1812,6 @@ _gth_file_list_update_next_thumb (GthFileList *file_list)
 		return;
 	}
 
-	file_store = (GthFileStore *) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
-
 	/* Find first visible undone. */
 
 	list = _gth_file_list_get_visibles (file_list);
@@ -1946,7 +1948,7 @@ _gth_file_list_exec_next_op (GthFileList *file_list)
 		gfl_set_files (file_list, op->file_list);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_ADD_FILES:
-		gfl_add_files (file_list, op->file_list);
+		gfl_add_files (file_list, op->file_list, op->position);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_DELETE_FILES:
 		gfl_delete_files (file_list, op->files);
